import {
  scaleBand,
  scaleLinear,
  scaleOrdinal,
  axisBottom,
  axisLeft,
  transition,
} from 'd3';
import { colors } from './utils';

export const barplot = (x) => {
  // Options
  let options = null;
  let labels = [];
  let dataset = [];
  
  // let width, margin, height;
  // if(options) {
  //   width = options.width;
  // 	height = options.height;
  // 	margin = options.margin;
  // }
  
  let width = 900;
  let height = 500;
  let margin = {
    top: 20,
    right: 20,
    bottom: 50,
    left: 50,
  };

  const my = (selection) => {
    // Transition
    const t = transition().duration(800);

    // Draw X-axis
    const xScale = scaleBand()
      .domain(labels)
      .padding(0.2)
      .range([margin.left, width - margin.right]);

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', '.x-axis')
      .attr(
        'transform',
        `translate(${0}, ${
          height - margin.bottom
        })`
      )
      .call(axisBottom(xScale));

    // Draw Y-axis
    const yScale = scaleLinear()
      .domain([0, 150])
      .range([
        height - margin.bottom,
        margin.top,
      ]);
    
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', '.y-axis')
      .attr(
        'transform',
        `translate(${margin.left}, ${0})`
      )
      .call(axisLeft(yScale));
    
    // X-SubGroup
    const xSubscale = scaleBand()
      .domain(dataset.map(d => d.label))
      .range([0, xScale.bandwidth()])
      .padding([0.05]);

    // Colors
    const colorScale = scaleOrdinal()
    	.domain(dataset.map(d => d.label))
    	.range(colors);
    
    // Process Data    
    const marks = labels.flatMap((d, i) => ({ 
      group: d, 
      data: dataset.map(d => ({
      	value: d.value[i],
        label: d.label
      }))
    }));
    
    // Draw Bars
    const bars = selection
      .selectAll('rect')
      .data(marks)
    	.join('g')
    		.attr('transform', d => `translate(${xScale(d.group)}, ${0})`)
    	.selectAll('rect')
    	.data((d) => d.data)
    	.join(
      	(enter) => {
          enter
          	.append('rect')
          	  .attr('x', (d) => xSubscale(d.label))
  						.attr('width', (d) => xSubscale.bandwidth())
  						.attr('fill', (d) => colorScale(d.label))
          		.attr('y', (d) => yScale(0))
  						.attr('height', 0)
          		.call((enter) => {
          			enter
                	.transition(t)
            			.attr('y', (d) => yScale(d.value))
  								.attr('height', (d) => height - yScale(d.value) - margin.bottom)
            			.delay((d, i) => i * 100)
          			}
              )
        },
        (update) => {},
        (exit) => {}
      );
    
    
  		// .join('rect')
  		// .attr('x', (d) => xSubscale(d.label))
  		// .attr('y', (d) => yScale(d.value))
  		// .attr('width', (d) => xSubscale.bandwidth())
  		// .attr('height', (d) => height - yScale(d.value) - margin.bottom)
  		// .attr('fill', (d) => colorScale(d.label))
  };

  // Getters & Setters
  my.options = function (_) {
    return arguments.length
      ? ((console.log(_)), (options = _), my)
      : options;
  };
  my.labels = function (_) {
    return arguments.length
      ? ((labels = _), my)
      : labels;
  };
  my.dataset = function (_) {
    return arguments.length
      ? ((dataset = _), my)
      : dataset;
  };

  return my;
};
